# -*- coding: utf-8 -*-

import unittest
import mock
import os
import json

from poormanci import consumer, models


def make_hub():
    hub = mock.Mock()
    hub.config = {
        'integrator.enabled': True,
    }
    return hub

SESSION = mock.Mock()


def make_session():
    return SESSION


PUNGI_CONFIG = mock.Mock()
PUNGI_CONFIG.configure_mock(
    name='pungi',
    display_name='Integration',
    pagure_name='pungi',
    pagure_url='https://pagure.io/',
    pagure_token='CAFEBABE',
    jenkins_name='pungi',
    jenkins_url='http://jenkins.fedorainfracloud.org/',
    jenkins_token='DEADBEEF',
)

TEST_PLAYGROUND_CONFIG = mock.Mock()
TEST_PLAYGROUND_CONFIG.configure_mock(
    name='test',
    pagure_name='lsedlar-test-playground',
    pagure_url='https://pagure.io/',
    pagure_token='CAFEBABE',
    jenkins_name='test-playground',
    jenkins_url='https://jenkins-lsedlar.rhcloud.com/',
    jenkins_token='DEADBEEF',
)


def not_found(name, service):
    raise models.ConfigNotFound()


def load_data(name):
    filename = os.path.join(os.path.dirname(__file__), 'data', name + '.json')
    with open(filename, 'r') as f:
        return json.load(f)


def load_msg(name):
    data = load_data(name)
    return {
        'topic': data['topic'],
        'body': data
    }


@mock.patch('poormanci.consumer.connect_db', make_session)
class ConsumerBuildTriggerTestCase(unittest.TestCase):

    @mock.patch('requests.post')
    @mock.patch('jenkins.Jenkins')
    @mock.patch('poormanci.models.create_run')
    @mock.patch('poormanci.models.get_configs')
    def test_can_submit_build_on_new_pr(self, get_configs, create_run, Jenkins, post):
        get_configs.return_value = [PUNGI_CONFIG]
        integrator = consumer.Integrator(make_hub())
        msg = load_msg('pull-request-new')

        integrator.consume(msg)

        self.assertEqual(get_configs.call_args_list,
                         [mock.call('pungi', models.Service.PAGURE)])
        self.assertEqual(create_run.call_args_list,
                         [mock.call(SESSION, 'pungi', 216)])
        self.assertEqual(Jenkins.return_value.call_args_list, [])
        self.assertEqual(post.call_args_list,
                         [mock.call(
                             'http://jenkins.fedorainfracloud.org/job/pungi/buildWithParameters',
                             {'REPO': 'https://pagure.io/forks/lsedlar/pungi.git',
                              'BRANCH': 'payload-fixup',
                              'BRANCH_TO': 'master',
                              'token': 'DEADBEEF',
                              'cause': create_run.return_value})])

    @mock.patch('requests.post')
    @mock.patch('jenkins.Jenkins')
    @mock.patch('poormanci.models.create_run')
    @mock.patch('poormanci.models.get_configs')
    def test_can_submit_build_on_updated_pr(self, get_configs, create_run, Jenkins, post):
        get_configs.return_value = [PUNGI_CONFIG]
        integrator = consumer.Integrator(make_hub())
        msg = load_msg('pull-request-updated')

        integrator.consume(msg)

        self.assertEqual(get_configs.call_args_list,
                         [mock.call('pungi', models.Service.PAGURE)])
        self.assertEqual(create_run.call_args_list,
                         [mock.call(SESSION, 'pungi', 213)])
        self.assertEqual(Jenkins.return_value.call_args_list, [])
        self.assertEqual(post.call_args_list,
                         [mock.call(
                             'http://jenkins.fedorainfracloud.org/job/pungi/buildWithParameters',
                             {'REPO': 'https://pagure.io/forks/lsedlar/pungi.git',
                              'BRANCH': 'unmodified-comps',
                              'BRANCH_TO': 'master',
                              'token': 'DEADBEEF',
                              'cause': create_run.return_value})])

    @mock.patch('requests.post')
    @mock.patch('jenkins.Jenkins')
    @mock.patch('poormanci.models.create_run')
    @mock.patch('poormanci.models.get_configs')
    def test_can_handle_unconfigured_project(self, get_configs, create_run, Jenkins, post):
        get_configs.side_effect = not_found
        integrator = consumer.Integrator(make_hub())
        msg = load_msg('pull-request-new-2')

        integrator.consume(msg)

        self.assertEqual(get_configs.call_args_list,
                         [mock.call('fedora-hubs', models.Service.PAGURE)])
        self.assertEqual(create_run.call_args_list, [])
        self.assertEqual(Jenkins.return_value.call_args_list, [])
        self.assertEqual(post.call_args_list, [])

    @mock.patch('requests.post')
    @mock.patch('jenkins.Jenkins')
    @mock.patch('poormanci.models.create_run')
    @mock.patch('poormanci.models.get_configs')
    def test_can_handle_random_comment(self, get_configs, create_run, Jenkins, post):
        integrator = consumer.Integrator(make_hub())
        msg = load_msg('pull-request-comment')
        integrator.consume(msg)

        self.assertEqual(get_configs.call_args_list, [])
        self.assertEqual(create_run.call_args_list, [])
        self.assertEqual(Jenkins.return_value.call_args_list, [])
        self.assertEqual(post.call_args_list, [])

    @mock.patch('requests.post')
    @mock.patch('jenkins.Jenkins')
    @mock.patch('poormanci.models.create_run')
    @mock.patch('poormanci.models.get_configs')
    def test_can_submit_build_from_main_repo(self, get_configs, create_run, Jenkins, post):
        get_configs.return_value = [TEST_PLAYGROUND_CONFIG]
        integrator = consumer.Integrator(make_hub())
        msg = load_msg('pull-request-from-main')

        integrator.consume(msg)

        self.assertEqual(get_configs.call_args_list,
                         [mock.call('lsedlar-test-playground', models.Service.PAGURE)])
        self.assertEqual(create_run.call_args_list,
                         [mock.call(SESSION, 'test', 6)])
        self.assertEqual(Jenkins.return_value.call_args_list, [])
        self.assertEqual(post.call_args_list,
                         [mock.call(
                             'https://jenkins-lsedlar.rhcloud.com/job/test-playground/buildWithParameters',
                             {'REPO': 'https://pagure.io/lsedlar-test-playground.git',
                              'BRANCH': 'test-pr',
                              'BRANCH_TO': 'master',
                              'token': 'DEADBEEF',
                              'cause': create_run.return_value})])

    @mock.patch('requests.post')
    @mock.patch('jenkins.Jenkins')
    @mock.patch('poormanci.models.create_run')
    @mock.patch('poormanci.models.get_configs')
    def test_can_submit_build_to_multiple_configs(self, get_configs, create_run, Jenkins, post):
        get_configs.return_value = [PUNGI_CONFIG, TEST_PLAYGROUND_CONFIG]
        integrator = consumer.Integrator(make_hub())
        msg = load_msg('pull-request-updated')
        create_run.side_effect = [12, 23]

        integrator.consume(msg)

        self.assertEqual(get_configs.call_args_list,
                         [mock.call('pungi', models.Service.PAGURE)])
        self.assertItemsEqual(create_run.call_args_list,
                              [mock.call(SESSION, 'pungi', 213),
                               mock.call(SESSION, 'test', 213)])
        self.assertEqual(Jenkins.return_value.call_args_list, [])
        self.assertItemsEqual(
            post.call_args_list,
            [mock.call('http://jenkins.fedorainfracloud.org/job/pungi/buildWithParameters',
                       {'REPO': 'https://pagure.io/forks/lsedlar/pungi.git',
                        'BRANCH': 'unmodified-comps',
                        'BRANCH_TO': 'master',
                        'token': 'DEADBEEF',
                        'cause': 12}),
             mock.call('https://jenkins-lsedlar.rhcloud.com/job/test-playground/buildWithParameters',
                       {'REPO': 'https://pagure.io/forks/lsedlar/pungi.git',
                        'BRANCH': 'unmodified-comps',
                        'BRANCH_TO': 'master',
                        'token': 'DEADBEEF',
                        'cause': 23})])


@mock.patch('poormanci.consumer.connect_db', make_session)
class ConsumerPostFlagTestCase(unittest.TestCase):

    @mock.patch('requests.post')
    @mock.patch('jenkins.Jenkins')
    @mock.patch('poormanci.models.mark_run')
    @mock.patch('poormanci.models.get_configs')
    def test_can_submit_success(self, get_configs, mark_run, Jenkins, post):
        get_configs.return_value = [PUNGI_CONFIG]
        integrator = consumer.Integrator(make_hub())
        msg = load_msg('jenkins-success')
        Jenkins.return_value.get_build_info.return_value = load_data('jenkins-build-success')
        mark_run.return_value.number = 216

        integrator.consume(msg)

        self.assertEqual(get_configs.call_args_list,
                         [mock.call('pungi', models.Service.JENKINS)])
        self.assertEqual(mark_run.call_args_list,
                         [mock.call(SESSION, 216, True, 119)])
        self.assertEqual(Jenkins.call_args_list,
                         [mock.call('http://jenkins.fedorainfracloud.org/')])
        self.assertEqual(Jenkins.return_value.get_build_info.call_args_list,
                         [mock.call('pungi', 119)])
        self.assertEqual(post.call_args_list,
                         [mock.call(
                             'https://pagure.io/api/0/pungi/pull-request/216/flag',
                             {'username': 'Integration',
                              'comment': 'Build successful',
                              'percent': 100,
                              'url': 'http://jenkins.fedorainfracloud.org/job/pungi/119/'},
                             headers={'Authorization': 'token CAFEBABE'})])

    @mock.patch('requests.post')
    @mock.patch('jenkins.Jenkins')
    @mock.patch('poormanci.models.mark_run')
    @mock.patch('poormanci.models.get_configs')
    def test_can_submit_fail(self, get_configs, mark_run, Jenkins, post):
        get_configs.return_value = [PUNGI_CONFIG]
        integrator = consumer.Integrator(make_hub())
        msg = load_msg('jenkins-fail')
        Jenkins.return_value.get_build_info.return_value = load_data('jenkins-build-fail')
        mark_run.return_value.number = 202

        integrator.consume(msg)

        self.assertEqual(get_configs.call_args_list,
                         [mock.call('pungi', models.Service.JENKINS)])
        self.assertEqual(mark_run.call_args_list,
                         [mock.call(SESSION, 202, False, 65)])
        self.assertEqual(Jenkins.call_args_list,
                         [mock.call('http://jenkins.fedorainfracloud.org/')])
        self.assertEqual(Jenkins.return_value.get_build_info.call_args_list,
                         [mock.call('pungi', 65)])
        self.assertEqual(post.call_args_list,
                         [mock.call(
                             'https://pagure.io/api/0/pungi/pull-request/202/flag',
                             {'username': 'Integration',
                              'comment': 'Build failed',
                              'percent': 0,
                              'url': 'http://jenkins.fedorainfracloud.org/job/pungi/65/'},
                             headers={'Authorization': 'token CAFEBABE'})])

    @mock.patch('requests.post')
    @mock.patch('jenkins.Jenkins')
    @mock.patch('poormanci.models.mark_run')
    @mock.patch('poormanci.models.get_configs')
    def test_can_handle_not_pr_check(self, get_configs, mark_run, Jenkins, post):
        get_configs.return_value = [PUNGI_CONFIG]
        integrator = consumer.Integrator(make_hub())
        msg = load_msg('jenkins-not-pr')
        Jenkins.return_value.get_build_info.return_value = load_data('jenkins-build-not-pr')

        integrator.consume(msg)

        self.assertEqual(get_configs.call_args_list,
                         [mock.call('pungi', models.Service.JENKINS)])
        self.assertEqual(mark_run.call_args_list, [])
        self.assertEqual(Jenkins.call_args_list,
                         [mock.call('http://jenkins.fedorainfracloud.org/')])
        self.assertEqual(Jenkins.return_value.get_build_info.call_args_list,
                         [mock.call('pungi', 118)])
        self.assertEqual(post.call_args_list, [])

    @mock.patch('requests.post')
    @mock.patch('jenkins.Jenkins')
    @mock.patch('poormanci.models.mark_run')
    @mock.patch('poormanci.models.get_configs')
    def test_can_handle_unknown_project(self, get_configs, mark_run, Jenkins, post):
        get_configs.side_effect = not_found
        integrator = consumer.Integrator(make_hub())
        msg = load_msg('jenkins-unknown')

        integrator.consume(msg)

        self.assertEqual(get_configs.call_args_list,
                         [mock.call('fedora-rube', models.Service.JENKINS)])
        self.assertEqual(mark_run.call_args_list, [])
        self.assertEqual(Jenkins.call_args_list,
                         [])
        self.assertEqual(Jenkins.return_value.get_build_info.call_args_list,
                         [])
        self.assertEqual(post.call_args_list, [])
